/*
 *
 * Copyright (C) 2007-2010 SimplyCore LLC
 * All rights reserved.
 * This file is part of USB IP project.
 * Distributed under Non-disclosure agreement.
 *
 */

#ifndef _USBD_FILTER_H_
#define _USBD_FILTER_H_

#ifndef __KERNEL__
#include <syslog.h>
#endif // __KERNEL__

#include "nettypes.h"

static int usbd_is_autosharing_allowed(
	const uint16 vid, 
	const uint16 pid, 
	const uint16 rev,
	const uint8 deviceClass, 
	const uint8 deviceSubClass, 
	const uint8 deviceProtocol, 
	const void * reserved)
{

    int rc;


    // Dump to debug
#ifdef __KERNEL__
    TRACE_CRITICAL( "Autoshare check for device.  vid=0x%04X, pid=0x%04X, rev=0x%04X, class=%i, subclass=%i, protocol=%i.\n", vid, pid, rev, deviceClass, deviceSubClass, deviceProtocol );
#else
    syslog( LOG_INFO, "Autoshare check for device.  vid=0x%04X, pid=0x%04X, rev=0x%04X, class=%i, subclass=%i, protocol=%i.\n", vid, pid, rev, deviceClass, deviceSubClass, deviceProtocol );
#endif // __KERNEL__
    
    rc = 1; // Allow by default
    
    // Never autoshare USB Keyboards, class:3 subclass:0 protocol:1
    // Never autoshare USB Mice     , class:3 subclass:0 protocol:2
    //  
    // Check for cetain HID class devices to prevent from autosharing
    if ( deviceClass == 3 ) 
        {   
        // Block Mice and Keyboards
        if (( deviceProtocol == 1 ) || ( deviceProtocol == 2 )) rc = 0;
        }   

    // Dump to debug
#ifdef __KERNEL__
    TRACE_CRITICAL( "Autoshare check result is %i.\n", rc );
#else
    syslog( LOG_INFO, "Autoshare check result is %i.\n", rc );
#endif
    
    return rc; 
}

#endif // _USBD_FILTER_H_
