/*
 *
 * Copyright (C) 2007-2010 SimplyCore LLC
 * All rights reserved.
 * This file is part of USB IP project.
 * Distributed under Non-disclosure agreement.
 *
 */

#ifndef _PUBSTRCT_
#define _PUBSTRCT_

#include <set>
#include <string>
#include <string.h>
#include "nettypes.h"

#pragma pack(push, 1)
typedef struct _taghwid {
	unsigned short idVendor;
	unsigned short idProduct;
	unsigned short bcdDevice;
} __attribute__((packed)) HWID, *PHWID;

typedef struct _tagiface {
	unsigned char Class;
	unsigned char SubClass;
	unsigned char Protocol;
} __attribute__((packed)) ZIFACE, *PZIFACE;
#pragma pack(pop)

#ifdef __cplusplus

	inline bool operator < (const struct _taghwid& usbhwid1,
				 const struct _taghwid& usbhwid2)
	{
		return (memcmp(&usbhwid1, &usbhwid2, sizeof(usbhwid1)) < 0);
	}

	inline bool operator == (const struct _taghwid& usbhwid1,
				 const struct _taghwid& usbhwid2)
	{
		return (memcmp(&usbhwid1, &usbhwid2, sizeof(usbhwid1)) == 0);
	}

	inline bool operator != (const struct _taghwid& usbhwid1,
				 const struct _taghwid& usbhwid2)
	{
		return (!memcmp(&usbhwid1, &usbhwid2, sizeof(usbhwid1)) == 0);
	}

#endif

//    .
typedef struct _md5handleinfo{
	char		md5[16];
	unsigned long	nHandle;
} MD5HANDLEINFO, *PMDHANDLE5INFO;

//    (hwid) USB 
typedef struct _tagdevprop {
	char	locationinfo[256];
	char	devdescription[256];
	char	hardwareid[256];
	char	compatibleid[256];
}DEVPROP, *PDEVPROP;

//       
typedef struct _tagtempusbinfo {
	uint32 hServer;
	uint32 hDevice;
	HWID			hwid;
	unsigned long	nDeviceProperties;
	std::string		busid;
	std::string		nickname;
	std::string		locinfo;
	std::string		devdesc;
	std::string		serial;

} TEMPUSBINFO, *PTEMPUSBINFO;

typedef struct _tagtempsrvinfo {

	uint32			hServer;
	std::string		sHostName;
	uint16			tcp_port;
	uint32			properties;

} TEMPSRVINFO, *PTEMPSRVINFO;

//    USB 
typedef struct _tagusbdevice {
	int				status;
	int				major;
	int				minor;
	int 			nDeviceFlags;
	char			nDeviceSpeed;
	void			*pclass;
	unsigned long	ip_client;
	unsigned long	clienthandle;
	HWID			hwid;
	ZIFACE			iface;
	std::string		busid;
	std::string		serialnumber;
	std::string		nickname;
	int				devprop_valid;
	DEVPROP			devprop;
}USBDEV, *PUSBDEV;

//   callback   
typedef struct _tagcallbackconnection {
	std::string	sHostName;
	uint8		nConnectionInProgress;
	uint16		tcp_port;
	uint32		nStatus;
	uint32		nProperties;
} CALLBACKCONNECTION, *PCALLBACKCONNECTION;

//   USB     
typedef struct _tagusbdevstate {
	int hDevice;
	int hServer;
	HWID hwid;
	ZIFACE iface;
	int	nDeviceStatus;
	int nDeviceFlags;
	char nDeviceSpeed;
	int ipclient;
	int nInactiveTimeout;
	std::string busid;
	std::string	locationinformation;
	std::string devicedescription;
	std::string	nickname;
	std::string serialnumber;
} USBDEVSTATE, *PUSBDEVSTATE;

//     
typedef struct _tagusbserverstate {
	uint32		hServer;
	uint32		nServerStatus;
	uint8		nPlatformId;
	std::string	sHostName;
	uint16		tcp_port;
	uint32		ulProperties;
} USBSERVERSTATE, *PUSBSERVERSTATE;

#endif // _PUBSTRCT_
